(self.webpackChunkttgtcanvas2=self.webpackChunkttgtcanvas2||[]).push([[480],{480:function(t,e,n){"use strict";var r=this&&this.__createBinding||(Object.create?function(t,e,n,r){void 0===r&&(r=n),Object.defineProperty(t,r,{enumerable:!0,get:function(){return e[n]}})}:function(t,e,n,r){void 0===r&&(r=n),t[r]=e[n]}),i=this&&this.__setModuleDefault||(Object.create?function(t,e){Object.defineProperty(t,"default",{enumerable:!0,value:e})}:function(t,e){t.default=e}),u=this&&this.__importStar||function(t){if(t&&t.__esModule)return t;var e={};if(null!=t)for(var n in t)"default"!==n&&Object.prototype.hasOwnProperty.call(t,n)&&r(e,t,n);return i(e,t),e};Object.defineProperty(e,"__esModule",{value:!0});const a=n(565),c=u(n(367)),o=n(657),s={id:"ttgtcanvas2:plugin",requires:[a.IJupyterWidgetRegistry],activate:function(t,e){e.registerWidget({name:o.MODULE_NAME,version:o.MODULE_VERSION,exports:c})},autoStart:!0};e.default=s}}]);